@@ -15,11 +15,12 @@ SQ=Snowflakeq()
 
 class Crp:
     """
+    CUSTOMER RETURN PROBABILITY MODULE USECASE
     """
 
 
 
-    def getCleanData(self,df):
+    def getCleanData(self,df:pd.DataFrame):
         """
         Params: 
             df-> Dataframe
@@ -39,19 +40,27 @@ class Crp:
         # Clean free orders 
         free = df[(df.totaldiscounts == df.totalorderitemsprice) & (df.totalprice == 0)]
         free_orders_num = len(free)
+
+        df_2 = df_1[~(df_1.id.isin(free['id'].values))]
+
+        # Missing customer data 
+        
+        missing_customers = len(df_2[(df_2['customerid'].isnull())])
+
+        df_3 = df_2[~(df_2['customerid'].isnull())] 
             
-        log_file.append([canceled_orders_num,free_orders_num])
+        log_file.append([canceled_orders_num,free_orders_num,missing_customers])
         
         # log file for documentation
-        log_file = pd.DataFrame(log_file,columns = ['Number_of_canceled_Orders','Number_of_free_orders'])
+        log_file = pd.DataFrame(log_file,columns = ['Number_of_canceled_Orders','Number_of_free_orders','Number_of_missing_customers_info'])
         
         
-        df_1 = df_1[~(df_1.id.isin(free['id'].values))]
-        
-        return df_1,log_file
+        display(log_file)
+        #output
+        return df_3
 
 
-    def getTrainingframe(self,df,dateFlag_sep,date_flag_min):
+    def getTrainingframe(self,df:pd.DataFrame,dateFlag_sep:str,date_flag_min:str):
         """
         Params:
             df                -> Dataframe
@@ -64,12 +73,12 @@ class Crp:
     #     dateFlag_sep = np.max(df['order_date']) - pd.DateOffset(months=timeFlag)
     #     date_flag_min = dateFlag_sep - pd.DateOffset(months=train_window)
         
-        df = df[(df.order_date <= dateFlag_sep) & (df.order_date >= date_flag_min)]
+        df = df[(df.order_date < dateFlag_sep) & (df.order_date >= date_flag_min)]
         
         return df 
 
 
-    def getTargetframe(self,df,dateFlag_sep,lag):
+    def getTargetframe(self,df:pd.DataFrame,dateFlag_sep:str,lag:int):
         """
         Params:
             df           -> dataframe
@@ -82,12 +91,12 @@ class Crp:
         """
     #     dateFlag_sep = np.max(df['order_date']) - pd.DateOffset(months=timeFlag)
         date_max = dateFlag_sep + pd.DateOffset(months=lag)
-        df = df[(df.order_date > dateFlag_sep) & (df.order_date < date_max)]
+        df = df[(df.order_date >= dateFlag_sep) & (df.order_date < date_max)]
         
         return df
 
 
-    def getTargetValues(self,train_data,target_data):
+    def getTargetValues(self,train_data:pd.DataFrame,target_data:pd.DataFrame):
         """
         Params:
             train_data  ->
@@ -110,7 +119,7 @@ class Crp:
         return target
 
 
-    def getRecency(self,matrix,dateFlag):
+    def getRecency(self,matrix:pd.DataFrame,dateFlag):
         """
         Params:
         
@@ -130,7 +139,7 @@ class Crp:
 
 
 
-    def getLastOrderparams(self,matrix,train_df):
+    def getLastOrderparams(self,matrix:pd.DataFrame,train_df:pd.DataFrame):
         """
         Params:
             matrix      ->
@@ -140,20 +149,22 @@ class Crp:
             output = matrix with last order value
         """
         
-        target = train_df[train_df.customerid.map(matrix['order_date']) == train_df.order_date][['customerid',
+        target = train_df[train_df.customerid.map(matrix['createdat']) == train_df.order_date][['customerid',
         'totalprice','totaldiscounts','totalweight']]
-        matrix = matrix.reset_index().rename(columns = {'totalprice':'LastOrderValue',
+        matrix = matrix.reset_index()
+
+        output = pd.merge(target,matrix,on='customerid',how='left')
+
+        output = output.rename(columns = {'totalprice':'LastOrderValue',
                                 'totaldiscounts':'LastOrderDiscount',
                                 'totalweight':'LastOrderWeight'})
-        output = pd.merge(target,matrix,on='customerid')
-        # output.drop_duplicates(inplace = True)
         
         # Output
         return output
 
 
 
-    def getTotalSpentLMonth(self,train_df,dateFlag_sep,df_matrix):
+    def getTotalSpentLMonth(self,train_df:pd.DataFrame,dateFlag_sep,df_matrix:pd.DataFrame):
         """
         Params:
 
@@ -165,7 +176,6 @@ class Crp:
 
         total_spent_last_month_sep = dateFlag_sep - pd.DateOffset(months = 1)
         
-        print('Last Month Money spent:',total_spent_last_month_sep)
         
         last_month_df = train_df[(train_df.order_date >= total_spent_last_month_sep - pd.DateOffset(months = 1)) & (train_df.order_date <= total_spent_last_month_sep)]
         
@@ -205,10 +215,12 @@ class Crp:
         # Create a grouping aggregator
         grp = train_df.groupby('customerid')
 
-        grp_dict = {'order_date':'max','id':'count','totalprice':'sum','buyeracceptsmarketing':'max',
+        grp_dict = {'createdat':'max','id':'count','totalprice':'sum','buyeracceptsmarketing':'max',
                     'billingaddresszip':'unique'}
 
         df_matrix = grp.agg(grp_dict).rename(columns={'id':'num_orders','totalprice':'TotalSpent'})
+
+        df_matrix['order_date'] = df_matrix['createdat'].dt.date
         # Recency Feature
         df_matrix = self.getRecency(df_matrix,dateFlag_sep)
         # Different Last orders parameters feature ( Discount value, weight ) 
@@ -224,7 +236,9 @@ class Crp:
 
         orders_param.columns = orders_param.columns.str.lower()
 
-        output = pd.merge(df_matrix,orders_param,on='customerid')
+        output = pd.merge(df_matrix,orders_param,on='customerid',how='left')
+
+        output = output.fillna(999)
         
         
         return output
@@ -247,7 +261,7 @@ class Crp:
         return output
 
 
-    def prepareDF(self,original_dataset,dateFlag_sep,date_flag_min,lag,items_dataset,customer_name,train_window,sq_pass):
+    def prepareDF(self,df,dateFlag_sep,date_flag_min,lag,items_dataset,customer_name,train_window,sq_pass):
         """
         Params:
             original_dataset ->
@@ -258,18 +272,13 @@ class Crp:
         :")
         """
         
-        # First, perform initial cleaning and save a log file of action
-    
-        df,log_file = self.getCleanData(original_dataset)
+
         # Get the first matrices
         train_data = self.getTrainingframe(df,dateFlag_sep,date_flag_min)
-        print('train_data Done')
         # For the target values calculations first -> get the relevant dataframe with the relevant timeframe
         target_frame = self.getTargetframe(df,dateFlag_sep,lag)
-        print('target frame done')
         # Calculate the target value
         target_value = self.getTargetValues(train_data,target_frame)
-        print('target value done')
         # Calculate features
         df = self.getFeaturesMatrix(train_data,original_dataset,lag,items_dataset,customer_name,train_window,sq_pass)
         
@@ -281,10 +290,10 @@ class Crp:
         
         # finalDf = finalDF.rename(columns = {'num_orders':'Num_orders','totalprice':'Total_spent'})
         
-        output =  finalDF
         
         
-        return output
+        
+        return finalDF
 
 
     def buildDF(self,original_dataset:pd.DataFrame,lag:int,train_window:int,customer_name,sq_pass,items_dataset:pd.DataFrame = None):
@@ -299,7 +308,6 @@ class Crp:
         
         # Lower limit date
         min_date = np.min(original_dataset['order_date']) + pd.DateOffset(months = train_window)
-        print('Min date for training timeframe:',min_date)
         output = []
         # Upper limit date
         date_max = np.max(original_dataset['order_date'])
@@ -307,16 +315,22 @@ class Crp:
         dateFlag_sep = date_max - pd.DateOffset(months=lag)
         # Initial timeframe min date
         date_flag_min = dateFlag_sep - pd.DateOffset(months=train_window)
-        print(dateFlag_sep,date_flag_min)
         # Construct dataframe according to that timeframe
-        output.append(self.prepareDF(original_dataset,dateFlag_sep,date_flag_min,lag,items_dataset,customer_name,train_window,sq_pass))
+
+
+        # First, perform initial cleaning and save a log file of action
+    
+        df= self.getCleanData(original_dataset)
+
+        
+        output.append(self.prepareDF(df,dateFlag_sep,date_flag_min,lag,items_dataset,customer_name,train_window,sq_pass))
         print(dateFlag_sep,date_flag_min)
         # Repeat the procedure until the bottom limit of the trainFrame reaches the minimum date possible according to the original dataset.
         while date_flag_min >= min_date:
         
             dateFlag_sep = dateFlag_sep - pd.DateOffset(months=lag)
             date_flag_min = dateFlag_sep - pd.DateOffset(months=train_window)
-            output.append(self.prepareDF(original_dataset,dateFlag_sep,date_flag_min,lag,items_dataset,customer_name,train_window,sq_pass))
+            output.append(self.prepareDF(df,dateFlag_sep,date_flag_min,lag,items_dataset,customer_name,train_window,sq_pass))
             print(dateFlag_sep,date_flag_min)
             
             
